﻿using dk.dake.Cryptography;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Security.Cryptography.X509Certificates;
using System.Text;

namespace SentinelApi
{
    public interface ISentinelAPI
    {
        #region 5.0 members
        ApiVersionResultat HentVersion();
        /// <summary>
        /// Genererer en systemafhængig tidsbegrænset engangsnøgle som skal benyttes til sytemlogind med kald til <see cref="SystemLogInd50"/>.
        /// </summary>
        /// <param name="systemnavn">sytem navn aftalt med sentinel.sundhed.dk</param>
        /// <param name="systemkodeord">sytemkodeord udleveret af sentinel.sundhed.dk</param>
        /// <returns>en tidsbegrænset engangsnøgle som i skal sendes dekrypteret i <see cref="SystemLogInd50"/></returns>
        ApiTokenResultat GenererSystemNoegle50(string systemnavn, string systemkodeord);

        /// <summary>
        /// Giver en systemlogindtoken, som benyttes i de øvrige API kald.
        /// Systemlogindtokenen virker kun d.d. og skal derfor fornys hver dag.
        /// Med denne systemlogintoken kan man kalde alle API kald, men hvis man kalder fra en webbaseret klient, 
        /// skal man af sikkerhedsårsager, generere en SystemKlientLogindToken istedet via kald til SystemKlientEngangsToken50 og SystemKlientLogInd50.
        /// 
        /// Inden man kan kalde dette skal:
        ///     1) Systemhuset  og Sentinel.dk have aftalt et <paramref name="systemnavn"/> for det enkelte praksissystem.
        ///     2) Systemhuset får udleveret et 'systemkodeord' som er bundet til ovenstående 'systemnavn'
        ///     3) Sentinel.Sundhed.dk have udleveret en public key til et certifikat som systemhuset vil bruge i forbindelse med dekryptering af <paramref name="encsystemnoegle"/>
        /// </summary>
        /// <param name="systemnavn">Det aftalte systemnavnt</param>
        /// <param name="encsystemnoegle">En krypteret sytemnøgle.
        ///  Systemnøglen, skal først hentes med  <see cref="GenererSystemNoegle50"/>, og skal derefter dekrypteres med privat nøglen til det aftalte system certifkat på følgende måde:
        ///  byte[] encryptedBytes = Convert.FromBase64String(systemnoegle);
        ///  byte[] decryptedBytes = privateKey.Decrypt(encryptedBytes, false);
        ///  string encsystemnoegle = Encoding.UTF8.GetString(decryptedBytes);
        ///  
        /// </param>
        /// <param name="systemversion">Versionen af det praksissystemet</param>
        /// <returns>En systemlogindtoken som kan deckrypteres og benyttes i <see cref="SystemLogInd50"/></returns>
        ApiTokenResultat SystemLogInd50(string systemnavn, string encsystemnoegle, string systemversion);

        /// <summary>
        /// Benyttes i forbindelse med generering af systemlogindtoken som kan benyttes i webklienter.
        /// returnerer en tidsbegrænset token som kan benyttes i kaldet <see cref="SystemKlientLogInd50"/>
        /// Dette kald skal foregå fra praksissystemets server. og den
        /// </summary>
        /// <param name="systemlogindtoken">En systemlogintoken genereret med </param>
        /// <param name="klientid"></param>
        /// <returns>en tidsbegranset engangstoken som benyttes i kald til <see cref="SystemKlientLogInd50"/> </returns>
        ApiTokenResultat GenererSystemKlientNoegle50(string systemlogindtoken, string klientid);

        /// <summary>
        /// Kaldes fra en webklient for at 'veksle' en engangstoken til en klientbundet systemlogindtoken, som benyttes i kald fra den påglende webklient.
        /// Kaldet skal foregå fra en webklient, hvor der skal laves ajax baserede kald til SentinelAPI'et.
        /// </summary>
        /// <param name="systemklientnoegle">token som er genereret med <see cref="GenererSystemKlientNoegle50"/></param>
        /// <returns>En systemlogindtoken, som er bundet til klientens browser, IP, klientid ...
        /// og derfor ikke kan 'stjæles' og benyttes fra en anden maskine</returns>
        ApiTokenResultat SystemKlientLogInd50(string systemklientnoegle);//bundet til klient og IP

        ApiResultat LogInd50(string systemlogindtoken,string klientid, string ydernr, string initialer);
        ApiResultat LogUd50(string systemlogindtoken, string klientid, string ydernr, string initialer);
        ApiResultat LogIndSession50(string systemlogindtoken, string klientid, string ydernr, string initialer,string sessionId);
        ApiResultat LogUdSession50(string systemlogindtoken, string klientid, string ydernr, string initialer,string sessionId);

        ApiResultat OpdaterStamdata50(string systemlogindtoken, string klientid, string ydernr, string initialer, string egenlaegeydernr, string egenlaegeinitialer, string cpr, string fornavn, string efternavn, string regionsKode, string emailadresse);

        ApiResultat IndsaetAnalyse50(string systemlogindtoken, string klientid, string lokalid, string haendelsestidspunkt, string ydernr, string initialer, string egenlaegeydernr, string egenlaegeinitialer, string cpr, string forloebsid, string kode, string kodeansvarlig, string kodebetydning, string producentKode, string producent, int _operator, string vaerdi, string enhed, string reffra, string reftil, string reftext, string abnorm, string proevetagningstidspunkt, string kommentarer);
        ApiResultat IndsaetDiagnose50(string systemlogindtoken, string klientid, string lokalid,  string haendelsestidspunkt, string ydernr, string initialer, string egenlaegeydernr, string egenlaegeinitialer, string cpr, string forloebsid, string forloebsrolle, string icpckode, string kodebetydning, string icpckilde, string icd10kode, string icd10kodebetydning, string icd10kilde);
        ApiResultat IndsaetMedicin50(string systemlogindtoken, string klientid, string lokalid, string haendelsestidspunkt, string ydernr, string initialer, string egenlaegeydernr, string egenlaegeinitialer, string cpr, string forloebsid, string varenummer, string atckode, string form, string navn, float styrke, string styrkeenhed, string styrketekst, string pakning, string doskode, string doskort, string doslang, int dosperiodeidage, string indKode, string indikation, int antalpakn, int antalite, int iteintervalidage);
        ApiResultat IndsaetYdelse50(string systemlogindtoken, string klientid, string lokalid, string haendelsestidspunkt, string ydernr, string initialer, string egenlaegeydernr, string egenlaegeinitialer, string cpr, string forloebsid, string ydelseskode, string ydelsestekst);
        ApiResultat IndsaetPost50(string systemlogindtoken, string klientid, string lokalid,  string ydernr, string initialer, bool udgaaende, string post, string forloebsid);
        ApiResultat IndsaetBehandling50(string systemlogindtoken, string klientid, string lokalid, string haendelsestidspunkt, string ydernr, string initialer, string egenlaegeydernr, string egenlaegeinitialer, string cpr, string forloebsid, string kode, string kodeansvarlig, string kodebetydning, string kommentarer);
        ApiResultat IndsaetNotat50(string systemlogindtoken, string klientid, string lokalid, string haendelsestidspunkt, string ydernr, string initialer, string egenlaegeydernr, string egenlaegeinitialer, string cpr, string forloebsid, string forloebsrolle, string notattype, string overskrift, string notat);

        ApiResultat RegistrerHaendelse50(string systemlogindtoken, string klientid, string haendelsestidspunkt, string ydernr, string initialer, string egenlaegeydernr, string egenlaegeinitialer, string cpr, string haendelseskode, string haendelsesdata);
        ApiResultat LukkerJournal50(string systemlogindtoken, string klientid, string ydernr, string initialer, string egenlaegeydernr, string egenlaegeinitialer, string cpr);
        ApiResultat AabnerJournal50(string systemlogindtoken, string klientid, string ydernr, string initialer, string egenlaegeydernr, string egenlaegeinitialer, string cpr);
        /// <summary>
        ///  SkifterJournal50 kan benyttes som alternativ til AabnerJournal50 og LukkerJournal50
        ///  SkifterJournal50 bør kun benyttes hvis man arbejder med højst 1 åben journal ad gangen.
        ///  SkifterJournal50 benyttes til at angive hvilken journal der aktuelt arbejdes på.
        ///  Kald til SkifterJournal50 svarer til at kalde LukkerJournal50 og AabnerJournal50 hvis og kun hvis den aktuelle journal ændres.
        ///  SkifterJournal50 kan derfor trygt kaldes flere gange med samme parametre, hvis man skulle være i tvivl om hvilken aktuel journal man sidst kommunikerede til Sentinel.
        ///  Kaldes SkifterJournal med et tomt cpr (null eller tom streng), svarer det til at kalde LukkerJournal50 på en evt. aktuel journal.
        ///  
        /// </summary>
        ApiResultat SkifterJournal50(string systemlogindtoken, string klientid, string ydernr, string initialer, string egenlaegeydernr, string egenlaegeinitialer, string cpr);

        ApiResultat SletAnalyse50(string systemlogindtoken, string klientid, string lokalid);
        ApiResultat SletMedicin50(string systemlogindtoken, string klientid, string lokalid);
        ApiResultat SletDiagnose50(string systemlogindtoken, string klientid, string lokalid);
        ApiResultat SletYdelse50(string systemlogindtoken, string klientid, string lokalid);
        ApiResultat SletPost50(string systemlogindtoken, string klientid, string lokalid);
        ApiResultat SletBehandling50(string systemlogindtoken, string klientid, string lokalid);
        ApiResultat SletNotat50(string systemlogindtoken, string klientid, string lokalid);

        ApiResultat LukSentinel50(string systemlogindtoken, string klientid);
       
        ApiMenuResultat HentMenu50(string systemlogindtoken, string klientid,string ydernr,string initialer,string cpr);
        
        ApiResultat UdfoerAktion50(string systemlogindtoken, string klientid, string aktion);

        ApiAktionerResultat HentAktioner50(string systemlogindtoken, string klientid, bool? autohaandtermeddelelser);
        ApiWebKlientResultat HentWebKlientUrl50(string systemlogindtoken, string klientid,bool? autohaandteraktioner);

        ApiTokenResultat HentKlientInfo50();
        ApiMenuResultat HentMenuPaaServer50(string systemlogindtoken, string klientinfotoken, string klientid, string ydernr, string initialer, string cpr);
        ApiAktionerResultat HentAktionerPaaServer50(string systemlogindtoken, string klientinfotoken, string klientid, bool? autohaandtermeddelelser);

        #endregion

        #region  5.1 members
        ApiResultat IndsaetDiagnose51(string systemlogindtoken, string klientid, string lokalid, string haendelsestidspunkt, string ydernr, string initialer, string egenlaegeydernr, string egenlaegeinitialer, string cpr, string forloebsid, string forloebsrolle, bool privat, string icpckode, string kodebetydning, string icpckilde, string icd10kode, string icd10kodebetydning, string icd10kilde);
        ApiResultat IndsaetForloeb51(string systemlogindtoken, string klientid, string lokalid, string haendelsestidspunkt, string ydernr, string initialer, string egenlaegeydernr, string egenlaegeinitialer, string cpr, string forloebsid,  bool privat, string sluttidspunkt);
        ApiResultat SletForloeb51(string systemlogindtoken, string klientid, string lokalid);

        #endregion

    }
}







